import os
import math
import arcade
import pymunk
import timeit
from PIL import Image

SCREEN_WIDTH = 1200
SCREEN_HEIGHT = 800
SCREEN_TITLE = "Angry Birds"

class PhysicsSprite(arcade.Sprite):
    def __init__(self, pymunk_shape, filename):
        super().__init__(filename, center_x=pymunk_shape.body.position.x, center_y=pymunk_shape.body.position.y)
        self.pymunk_shape = pymunk_shape

class CircleSprite(PhysicsSprite):
    def __init__(self, pymunk_shape, filename):
        super().__init__(pymunk_shape, filename)
        self.width = pymunk_shape.radius * 2
        self.height = pymunk_shape.radius * 2

class BoxSprite(PhysicsSprite):
    def __init__(self, pymunk_shape, filename, width, height):
        super().__init__(pymunk_shape, filename)
        self.width = width
        self.height = height
 
def make_sprite(mass,image,position,space):      
    width,height = Image.open(image).size  
    mass = mass
    moment = pymunk.moment_for_box(mass, (width,height))
    body = pymunk.Body(mass, moment)
    body.position = pymunk.Vec2d(position)
    shape = pymunk.Poly.create_box(body, (width, height))
    shape.friction = 0.3
    space.add(body, shape)
    sprite = BoxSprite(shape, image, width=width, height=height)
    return sprite       


class MyGame(arcade.Window):

    def __init__(self, width, height,title):
        super().__init__(width, height,title)
        
        arcade.set_background_color(arcade.color.DARK_SLATE_GRAY)

        self.parabolic_points = []
        self.dx = self.dy = 0
 
        self.space = pymunk.Space()
        self.space.gravity = (0.0, -900.0)

        self.background = arcade.Sprite("images/background.png")
        self.background.left = self.background.bottom = 0
        self.sprite_list = arcade.SpriteList()
        self.static_lines = []

        self.shape_being_dragged = None
        self.last_mouse_position = 0, 0

        self.draw_time = 0
        self.processing_time = 0

        floor_height = 80
        body = pymunk.Body(body_type=pymunk.Body.STATIC)
        shape = pymunk.Segment(body, [0, floor_height], [SCREEN_WIDTH, floor_height], 0.0)
        shape.friction = 10
        self.space.add(shape)
        self.static_lines.append(shape)

        box = make_sprite(43,"images/boxCrate_double.png",(1000,  130),self.space)
        self.sprite_list.append(box) 
        box = make_sprite(43,"images/boxCrate_double.png",(1000,  230),self.space)
        self.sprite_list.append(box) 
        box = make_sprite(43,"images/boxCrate_double.png",(1000,  330),self.space)
        self.sprite_list.append(box) 
        box = make_sprite(43,"images/boxCrate_double.png",(1000,  430),self.space)
        self.sprite_list.append(box) 
        box = make_sprite(43,"images/boxCrate_double.png",(1000,  530),self.space)
        self.sprite_list.append(box)          
  
        sprite = make_sprite(33,"images/pig.png",(1000, 620),self.space)
        self.sprite_list.append(sprite)        

        sprite = make_sprite(33,"images/platform.png",(160, 160),self.space)
        self.sprite_list.append(sprite)        

        self.shoot_position = pymunk.Vec2d(160 , 290)             
        self.physic_bird = make_sprite(33,"images/bird.png",self.shoot_position,self.space)
        self.sprite_list.append(self.physic_bird)        
        
        self.virtual_bird = arcade.Sprite("images/bird.png")
        self.virtual_bird.center_x = self.shoot_position[0]
        self.virtual_bird.center_y = self.shoot_position[1]  

        self.reset_shoot = True
        
    def reset_shoot_bird(self):
        self.virtual_bird.center_x = self.shoot_position[0]
        self.virtual_bird.center_y = self.shoot_position[1]
        self.physic_bird.pymunk_shape.body.velocity = (0,0)
        self.physic_bird.pymunk_shape.body.position = self.shoot_position
        self.physic_bird.pymunk_shape.body.angle = 0 
        
        self.reset_shoot = True
        self.shape_being_dragged = None
        
    def on_key_press(self, key, modifiers):

        if key == arcade.key.SPACE:
           self.reset_shoot_bird()       
        
         
    def on_draw(self):

        arcade.start_render()

        draw_start_time = timeit.default_timer()

        self.background.draw()

        self.sprite_list.draw()

        for line in self.static_lines:
            body = line.body
            pv1 = body.position + line.a.rotated(body.angle)
            pv2 = body.position + line.b.rotated(body.angle)
            arcade.draw_line(pv1.x, pv1.y, pv2.x, pv2.y, arcade.color.WHITE, 2)

        if self.reset_shoot :  self.virtual_bird.draw()

        x1,y1 = self.shoot_position
        x2,y2 = self.virtual_bird.position
        dd = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)
         
        if dd>2 and self.reset_shoot:
          arcade.draw_line(x1, y1, x2, y2, arcade.color.ORANGE, 4)

    def on_mouse_press(self, x, y, button, modifiers): 
        
        if button == arcade.MOUSE_BUTTON_LEFT :  
            self.shape_being_dragged = self.virtual_bird 

    def on_mouse_release(self, x, y, button, modifiers):
        
        physic_bird_position = self.physic_bird.pymunk_shape.body.position 
        distance = (physic_bird_position.get_distance(self.shoot_position)) 
        if distance > 1 :return
        
        if button == arcade.MOUSE_BUTTON_LEFT:
            self.shape_being_dragged = None              
            self.dx = x - self.shoot_position[0]
            self.dy = y - self.shoot_position[1]
            print("Emission",self.dx,self.dy)
            self.physic_bird.pymunk_shape.body.force = -self.dx*20000, -self.dy*20000
            self.reset_shoot = False
            self.dx = 0
            self.dy = 0
            self.parabolic_points = []

    def on_mouse_motion(self, x, y, dx, dy):
        if self.shape_being_dragged is not None:
            
            self.virtual_bird.center_x = x
            self.virtual_bird.center_y = y
            self.dx = x - self.shoot_position[0]
            self.dy = y - self.shoot_position[1]

            print("self.dx,self.dy",self.dx,self.dy)
            gx,gy = self.space.gravity
            gy = -gy/100
            self.parabolic_points = []
            x , y = self.shoot_position
        
            for i in range(20):                   
                x = x - self.dx
                y = y - self.dy
                self.dy = self.dy + gy                            
                self.parabolic_points.append((x,y))                       

    def update(self, delta_time):
         
        start_time = timeit.default_timer()        
        self.virtual_bird.update()
        
        for sprite in self.sprite_list:
            if self.physic_bird == sprite:continue         
            if sprite.pymunk_shape.body.position.y < 0:
                self.space.remove(sprite.pymunk_shape, sprite.pymunk_shape.body)
                sprite.kill()

        self.space.step(1 / 60.0)
 
        for sprite in self.sprite_list:
            sprite.center_x = sprite.pymunk_shape.body.position.x
            sprite.center_y = sprite.pymunk_shape.body.position.y
            sprite.angle = math.degrees(sprite.pymunk_shape.body.angle)

        self.processing_time = timeit.default_timer() - start_time
         

def main():
    MyGame(SCREEN_WIDTH, SCREEN_HEIGHT,SCREEN_TITLE)

    arcade.run()

if __name__ == "__main__":
    main()
